-------------------------------------------------------------------------------
-- DISCLAIMER OF LIABILITY
--
-- DLP Design is providing this design, code, or information "as-is" solely for 
-- use in developing programs and solutions for DLP Design products, with no 
-- obligation on the part of DLP Design to provide support. DLP Design expressly 
-- disclaims any warranty whatsoever with respect to the adequacy of the 
-- implementation, including but not limited to any warranties or representations 
-- that this implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular purpose.
--
-- DLP Design products are not intended for use in life support appliances, devices, 
-- or systems. Use in such applications is expressly prohibited.
--
-- Any modifications that are made to the Source Code are done at the user's sole 
-- risk and will be unsupported.
--
-- Copyright (c) 2006-2009 DLP Design, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part of this text at all times.
----------------------------------------------------------------------------------
-- Company: DLP Design, Inc., www.dlpdesign.com
-- 
-- Create Date:    18:24:10 03/13/2006 
-- Design Name: 	 usb_test	
-- Module Name:    demo - Behavioral 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S250E-4TQ144A
-- Tool versions:  ISE 9.1.03i
-- Description:    top module for mfg test design
--                 
-- Dependencies:   submodules startup, lowdcm, highdcm, timer,
--                 heartbeat, usb_state_machine, cmd_exec, demo.ucf
--
-- Revision: 
-- 	0.01 	03/13/06	File Created for demo function
--	0.02	12/15/06 	Stripped out mother board peripherals
--                      so that USB pin assert and pin read
--                      functions could be added for mfg test
--    0.04  12/27/06 	Added usb state machine component
--	0.05	12/28/06 	Added IO bidirectional buffers
--    0.06  02/14/07 	Added command exec component
--    0.07  02/28/07 	Changed cmd_exec clock to 6 MHz
--    1.00  07/31/07	Final release
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- The following library declaration required since we are instantiating Xilinx primitives in this code.

library UNISIM;
use UNISIM.VComponents.all;

entity demo is
    Port ( clk_6m : in STD_LOGIC;
           resetn : in STD_LOGIC;
			  user_in : in std_logic_vector(10 downto 0);
			  --button(0) = USER_IN2 = SW1 on DLP-FPGA-LAB
			  --button(1) = USER_IN1 = SW2 on DLP-FPGA-LAB
			  --button : in std_logic_vector(1 downto 0);
			  --switch(0) = USER_IN3 = SW4-1 on DLP-FPGA-LAB
			  --switch(1) = USER_IN4 = SW4-2 on DLP-FPGA-LAB
			  --switch(2) = USER_IN5 = SW4-3 on DLP-FPGA-LAB
			  --switch(3) = USER_IN6 = SW4-4 on DLP-FPGA-LAB
			  --switch(4) = USER_IN7 = SW4-5 on DLP-FPGA-LAB
			  --switch(5) = USER_IN8 = SW4-6 on DLP-FPGA-LAB
			  --switch(6) = USER_IN9 = SW4-7 on DLP-FPGA-LAB
			  --switch(7) = USER_IN10 = SW4-8 on DLP-FPGA-LAB
			  --switch : in std_logic_vector(7 downto 0);
			  sram_a : out std_logic_vector(16 downto 0);
			  sram_d : inout std_logic_vector(7 downto 0);
			  sram_oe : out std_logic;
			  sram_we : out std_logic;
			  ftdi_d : inout std_logic_vector(7 downto 0);
			  ftdi_rxf_n : in std_logic;
			  ftdi_txe_n : in std_logic;
			  ftdi_rd_n : out std_logic;
			  ftdi_wr_n : out std_logic;
			  user_io_u : inout std_logic_vector(28 downto 16);
			  user_ip : in std_logic_vector(15 downto 14);
			  user_io_l : inout std_logic_vector(13 downto 0);
			  --lcd_rs : out std_logic;
			  --lcd_en : out std_logic;
			  --lcd_d : out std_logic_vector(7 downto 0);
			  --ledg_mb : out std_logic;
			  --ledr_mb : out std_logic;
           ledr_heartbeat : out  STD_LOGIC);
end demo;

architecture Behavioral of demo is

	COMPONENT startup
	PORT(
		clk_in : IN std_logic;
		rst_in : IN std_logic;
		switch : IN std_logic_vector(7 downto 0);          
		start : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
	
	COMPONENT lowdcm -- DCM #1
	PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKDV_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT timer
	PORT(
		clkin_2u5s : IN std_logic;
		ce_in : IN std_logic;
		rst_in : IN std_logic;          
		clk_10us : OUT std_logic;
		clk_80us : OUT std_logic;
		clk_1m28s : OUT std_logic;
		clk_5m12s : OUT std_logic;
		clk_10m24s : OUT std_logic;
		clk_40m96s : OUT std_logic;
		clk_81m92s : OUT std_logic;
		clk_655m36s : OUT std_logic
		);
	END COMPONENT;

	COMPONENT heartbeat
	PORT(
		clk_in : IN std_logic;
		rst_in : IN std_logic;          
		ledr_heartbeat : OUT std_logic
		);
	END COMPONENT;
	
   COMPONENT highdcm -- DCM #2
   port ( CLKIN_IN   : in    std_logic; 
          RST_IN     : in    std_logic; 
          CLKDV_OUT  : out   std_logic; 
          CLKFX_OUT  : out   std_logic; 
          CLK0_OUT   : out   std_logic; 
          CLK2X_OUT  : out   std_logic; 
          LOCKED_OUT : out   std_logic);
	END COMPONENT;

	COMPONENT usb_state_machine
   GENERIC(	
		BOARD_ID  : std_logic_vector(7 downto 0):=x"01";  -- 1=initial proto
      FPGA_ID   : std_logic_vector(7 downto 0):=x"01";  -- 1=XC3S250E-4TQ144CES
	   DESIGN_ID : std_logic_vector(7 downto 0):=x"01"); -- 1=initial design
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		RXF_n : IN std_logic;
		TXE_n : IN std_logic;
		ftdi_din : IN std_logic_vector(7 downto 0);
		local_done : IN std_logic;
		local_din : IN std_logic_vector(7 downto 0);          
		ftdi_rd : OUT std_logic;
		ftdi_wr : OUT std_logic;
		ftdi_dir : OUT std_logic;
		ftdi_dout : OUT std_logic_vector(7 downto 0);
		local_ready : OUT std_logic;
		local_dout : OUT std_logic_vector(7 downto 0);
		ram_addr : OUT std_logic_vector(16 downto 0);
		ram_data : OUT std_logic_vector(7 downto 0);
		pinget : OUT std_logic;
		pinset : OUT std_logic;
		pinclr : OUT std_logic;
		r_ram : OUT std_logic;
		w_ram : OUT std_logic
		);
	END COMPONENT;

	COMPONENT cmd_exec
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		cmd_ready : IN std_logic;
		cmd_in : IN std_logic_vector(7 downto 0);
		r_ram : IN std_logic;
		w_ram : IN std_logic;
		ram_address : IN std_logic_vector(16 downto 0);
		ram_data : IN std_logic_vector(7 downto 0);
		din : IN std_logic_vector(7 downto 0);
		pinget : IN std_logic;
		pinset : IN std_logic;
		pinclr : IN std_logic;
		user_io_in : IN std_logic_vector(28 downto 0);
		user_in : IN std_logic_vector(10 downto 0);          
		exec_done : OUT std_logic;
		result_out : OUT std_logic_vector(7 downto 0);
		a : OUT std_logic_vector(16 downto 0);
		dout : OUT std_logic_vector(7 downto 0);
		dir : OUT std_logic;
		oe : OUT std_logic;
		we : OUT std_logic;
		user_io_out : OUT std_logic_vector(28 downto 0);
		user_io_dir : OUT std_logic_vector(28 downto 0)
		);
	END COMPONENT;

	constant BOARD_ID  : std_logic_vector(7 downto 0):=x"12";--x"01";  -- 1=initial proto
   constant FPGA_ID   : std_logic_vector(7 downto 0):=x"34";--x"01";  -- 1=XC3S250E-4TQ144CES
	constant DESIGN_ID : std_logic_vector(7 downto 0):=x"56";--x"01";  -- 1=initial design

   --signal resetn : STD_LOGIC; ---***debug
	
	signal rst,resetp_1,resetp_2 : std_logic;
	signal resetn_1,resetn_2 : std_logic;
	signal locked1,locked2 : std_logic;
	signal clk_6m_int : std_logic;
	signal clk_6m_dll,clk_6m_dll2 : std_logic;
	signal clk_12m,clk_50m : std_logic;
	signal clk_400k,clk_1m714 : std_logic;
	signal clk_10us : std_logic;
	signal clk_80us : std_logic;
	signal clk_1m28s : std_logic;
	signal clk_5m12s : std_logic;
	signal clk_10m24s : std_logic;
	signal clk_40m96s : std_logic;
	signal clk_81m92s : std_logic;
	signal clk_655m36s : std_logic; 
   signal sram_dout : std_logic_vector(7 downto 0);
   signal sram_din : std_logic_vector(7 downto 0);
	signal dir,ftdi_dir,local_ready, local_done : std_logic;
   signal ftdi_dout : std_logic_vector(7 downto 0);
   signal ftdi_din : std_logic_vector(7 downto 0);
   signal start : std_logic_vector(7 downto 0);
   signal local_din,local_dout,ram_data : STD_LOGIC_VECTOR (7 downto 0);
   signal ram_addr : STD_LOGIC_VECTOR (16 downto 0);
   signal pinget,pinset,pinclr,r_ram,w_ram : std_logic;
	signal user_io_in,user_io_out : std_logic_vector(28 downto 0);
	signal user_io_cat,user_io_dir : std_logic_vector(28 downto 0);

begin

	--==================================================================
	-- Top Level Combinatorial assignments
	--==================================================================

   --resetn <= '1'; --***debug
	
   -- reset signal for DCMs
	rst <= '1' when (resetn = '0') else '0';
	--rst <= '0';

   -- reset signal for blocks driven by lowdcm
	resetp_1 <= '0' when locked1 = '1' else '1';	-- active hi            
	resetn_1 <= '1' when locked1 = '1' else '0';	-- active lo            

   -- reset signal for blocks driven by highdcm
	resetp_2 <= '0' when locked2 = '1' else '1';	-- active hi	            
	resetn_2 <= '1' when locked2 = '1' else '0';	-- active lo	            

	-- ledg_mb driven by button(0) SW1 '0' is on
	--ledg_mb <= '0' when (button(0) = '0' or button(1) = '0') else '1';

	--ledg_mb <= '0' when (sram_errors = X"00000") else '1';
	-- ledg_mb driven by button(0) SW1 '0' is on
	--ledr_mb <= '0' when button(1) = '0' else '1';

	--ledr_mb <= '0' when (sram_errors = X"00000") else '1';
	
	-- map inputs to cat vector to allow IP 14&15 to be read
	user_io_cat(0) <= user_io_in(0);
	user_io_cat(1) <= user_io_in(1);
	user_io_cat(2) <= user_io_in(2);
	user_io_cat(3) <= user_io_in(3);
	user_io_cat(4) <= user_io_in(4);
	user_io_cat(5) <= user_io_in(5);
	user_io_cat(6) <= user_io_in(6);
	user_io_cat(7) <= user_io_in(7);
	user_io_cat(8) <= user_io_in(8);
	user_io_cat(9) <= user_io_in(9);
	user_io_cat(10) <= user_io_in(10);
	user_io_cat(11) <= user_io_in(11);
	user_io_cat(12) <= user_io_in(12);
	user_io_cat(13) <= user_io_in(13);
	user_io_cat(14) <= user_ip(14);
	user_io_cat(15) <= user_ip(15);
	user_io_cat(16) <= user_io_in(16);
	user_io_cat(17) <= user_io_in(17);
	user_io_cat(18) <= user_io_in(18);
	user_io_cat(19) <= user_io_in(19);
	user_io_cat(20) <= user_io_in(20);
	user_io_cat(21) <= user_io_in(21);
	user_io_cat(22) <= user_io_in(22);
	user_io_cat(23) <= user_io_in(23);
	user_io_cat(24) <= user_io_in(24);
	user_io_cat(25) <= user_io_in(25);
	user_io_cat(26) <= user_io_in(26);
	user_io_cat(27) <= user_io_in(27);
	user_io_cat(28) <= user_io_in(28);

	
	--==================================================================
	-- Top level Component Instantiations
	--==================================================================

	Inst_startup: startup PORT MAP(
		clk_in => clk_50m,
		rst_in => resetp_2, --high dcm locked
		switch => user_in(10 downto 3), --switch,
		start => start 
	);
	
	Inst_lowdcm: lowdcm PORT MAP(
		CLKIN_IN => clk_6m,  -- 167 ns period
		RST_IN => rst,
		CLKDV_OUT => clk_400k,  -- 2.5 us period
		CLKIN_IBUFG_OUT => clk_6m_int,
		CLK0_OUT => clk_6m_dll,
		LOCKED_OUT => locked1 
	);

	Inst_timer: timer PORT MAP(
		clkin_2u5s => clk_400k,
		ce_in => locked1,
		rst_in => rst,
		clk_10us => clk_10us,
		clk_80us => clk_80us,
		clk_1m28s => clk_1m28s,
		clk_5m12s => clk_5m12s,
		clk_10m24s => clk_10m24s,
		clk_40m96s => clk_40m96s,
		clk_81m92s => clk_81m92s,
		clk_655m36s => clk_655m36s 
	);

	Inst_heartbeat: heartbeat PORT MAP(
		clk_in => clk_655m36s,  -- 0.65536 sec period
		rst_in => resetp_1,
		ledr_heartbeat => ledr_heartbeat
	);
	
	Inst_highdcm: highdcm PORT MAP(
		CLKIN_IN => clk_6m_int, -- 167 ns period
		RST_IN => rst,
		CLKFX_OUT => clk_50m, -- 20 ns period, could go up to 75 MHz (13 ns)
		CLKDV_OUT => clk_1m714,  -- 583 ns period
		CLK0_OUT => clk_6m_dll2,
		CLK2X_OUT => clk_12m, -- 83 ns period
		LOCKED_OUT => locked2
	);	

	Inst_usb_state_machine: usb_state_machine 
   GENERIC MAP(	
		BOARD_ID  => BOARD_ID,  
      FPGA_ID   => FPGA_ID,  
	   DESIGN_ID => DESIGN_ID)  
	PORT MAP(
		clk => clk_50m,
		reset => resetp_2,
		RXF_n => ftdi_rxf_n,
		TXE_n => ftdi_txe_n,
		ftdi_rd => ftdi_rd_n,
		ftdi_wr => ftdi_wr_n,
		ftdi_dir => ftdi_dir,
		ftdi_din => ftdi_din,
		ftdi_dout => ftdi_dout,
		local_ready => local_ready,
		local_done => local_done,
		local_din => local_din,
		local_dout => local_dout,
		ram_addr => ram_addr,
		ram_data => ram_data,
		pinget => pinget,
		pinset => pinset,
		pinclr => pinclr,
		r_ram => r_ram,
		w_ram => w_ram
	);
	
	Inst_cmd_exec: cmd_exec PORT MAP(
		clk => clk_6m_dll, --clk_50m,
		reset => resetp_2,
		cmd_ready => local_ready, -- pin/ram access requested
		exec_done => local_done, -- pin/ram access completed
		cmd_in => local_dout, -- data from usb, selected pin value
		result_out => local_din, -- data to usb
		r_ram => r_ram, -- ram read requested
		w_ram => w_ram, -- ram write requested
		ram_address => ram_addr,
		ram_data => ram_data,
		a => sram_a,
		din => sram_din,
		dout => sram_dout,
		dir => dir,
		oe => sram_oe,
		we => sram_we,
		pinget => pinget,
		pinset => pinset,
		pinclr => pinclr,
		user_io_in => user_io_cat,
		user_io_out => user_io_out,
		user_io_dir => user_io_dir,
		user_in => user_in
	);

	--==================================================================
	-- Tristate Buffers
	--==================================================================

	---------------- SRAM data bus tristate buffers --------------------

   inst_IOBUF0 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => sram_din(0),   -- Buffer output
      IO => sram_d(0),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(0),  -- Buffer input
      T => dir            -- 3-state enable input 
   );
 
   inst_IOBUF1 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(1),   -- Buffer output
      IO => sram_d(1),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(1),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF2 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(2),   -- Buffer output
      IO => sram_d(2),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(2),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF3 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(3),   -- Buffer output
      IO => sram_d(3),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(3),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF4 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(4),   -- Buffer output
      IO => sram_d(4),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(4),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF5 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(5),   -- Buffer output
      IO => sram_d(5),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(5),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF6 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(6),   -- Buffer output
      IO => sram_d(6),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(6),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

   inst_IOBUF7 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVTTL",
      SLEW => "FAST")
   port map (
      O => sram_din(7),   -- Buffer output
      IO => sram_d(7),    -- Buffer inout port (connect directly to top-level port)
      I => sram_dout(7),  -- Buffer input
      T => dir            -- 3-state enable input 
   );

	------------------ USB data bus tristate buffers ---------------------

   inst_IOBUF8 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(0),   -- Buffer output
      IO => ftdi_d(0),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(0),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF9 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(1),   -- Buffer output
      IO => ftdi_d(1),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(1),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF10 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(2),   -- Buffer output
      IO => ftdi_d(2),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(2),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF11 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(3),   -- Buffer output
      IO => ftdi_d(3),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(3),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF12 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(4),   -- Buffer output
      IO => ftdi_d(4),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(4),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF13 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(5),   -- Buffer output
      IO => ftdi_d(5),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(5),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF14 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(6),   -- Buffer output
      IO => ftdi_d(6),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(6),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

   inst_IOBUF15 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "FAST")
   port map (
      O => ftdi_din(7),   -- Buffer output
      IO => ftdi_d(7),    -- Buffer inout port (connect directly to top-level port)
      I => ftdi_dout(7),  -- Buffer input
      T => ftdi_dir       -- 3-state enable input 
   );

	------------------ USER IO tristate buffers ---------------------

   inst_IOBUF16 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(0),   -- Buffer output
      IO => user_io_l(0),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(0),  -- Buffer input
      T => user_io_dir(0)   -- 3-state enable input 
   );

   inst_IOBUF17 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(1),   -- Buffer output
      IO => user_io_l(1),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(1),  -- Buffer input
      T => user_io_dir(1)   -- 3-state enable input 
   );

   inst_IOBUF18 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(2),   -- Buffer output
      IO => user_io_l(2),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(2),  -- Buffer input
      T => user_io_dir(2)   -- 3-state enable input 
   );

   inst_IOBUF19 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(3),   -- Buffer output
      IO => user_io_l(3),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(3),  -- Buffer input
      T => user_io_dir(3)   -- 3-state enable input 
   );

   inst_IOBUF20 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(4),   -- Buffer output
      IO => user_io_l(4),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(4),  -- Buffer input
      T => user_io_dir(4)   -- 3-state enable input 
   );

   inst_IOBUF21 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(5),   -- Buffer output
      IO => user_io_l(5),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(5),  -- Buffer input
      T => user_io_dir(5)   -- 3-state enable input 
   );

   inst_IOBUF22 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(6),   -- Buffer output
      IO => user_io_l(6),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(6),  -- Buffer input
      T => user_io_dir(6)   -- 3-state enable input 
   );

   inst_IOBUF23 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(7),   -- Buffer output
      IO => user_io_l(7),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(7),  -- Buffer input
      T => user_io_dir(7)   -- 3-state enable input 
   );

   inst_IOBUF24 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(8),   -- Buffer output
      IO => user_io_l(8),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(8),  -- Buffer input
      T => user_io_dir(8)   -- 3-state enable input 
   );

   inst_IOBUF25 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(9),   -- Buffer output
      IO => user_io_l(9),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(9),  -- Buffer input
      T => user_io_dir(9)   -- 3-state enable input 
   );

   inst_IOBUF26 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(10),   -- Buffer output
      IO => user_io_l(10),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(10),  -- Buffer input
      T => user_io_dir(10)   -- 3-state enable input 
   );

   inst_IOBUF27 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(11),   -- Buffer output
      IO => user_io_l(11),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(11),  -- Buffer input
      T => user_io_dir(11)   -- 3-state enable input 
   );

   inst_IOBUF28 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(12),   -- Buffer output
      IO => user_io_l(12),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(12),  -- Buffer input
      T => user_io_dir(12)   -- 3-state enable input 
   );

   inst_IOBUF29 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(13),   -- Buffer output
      IO => user_io_l(13),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(13),  -- Buffer input
      T => user_io_dir(13)   -- 3-state enable input 
   );

-- user io 14 & 15 are actually input only pins

--   inst_IOBUF30 : IOBUF
--   generic map (
--      DRIVE => 12,
--      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
--      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
--      IOSTANDARD => "LVCMOS33",
--      SLEW => "SLOW")
--   port map (
--      O => user_io_in(14),   -- Buffer output
--      IO => user_io(14),     -- Buffer inout port (connect directly to top-level port)
--      I => user_io_out(14),  -- Buffer input
--      T => user_io_dir(14)   -- 3-state enable input 
--   );

--   inst_IOBUF31 : IOBUF
--   generic map (
--      DRIVE => 12,
--      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
--      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
--      IOSTANDARD => "LVCMOS33",
--      SLEW => "SLOW")
--   port map (
--      O => user_io_in(15),   -- Buffer output
--      IO => user_io(15),     -- Buffer inout port (connect directly to top-level port)
--      I => user_io_out(15),  -- Buffer input
--      T => user_io_dir(15)   -- 3-state enable input 
--   );

   inst_IOBUF32 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(16),   -- Buffer output
      IO => user_io_u(16),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(16),  -- Buffer input
      T => user_io_dir(16)   -- 3-state enable input 
   );

   inst_IOBUF33 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(17),   -- Buffer output
      IO => user_io_u(17),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(17),  -- Buffer input
      T => user_io_dir(17)   -- 3-state enable input 
   );

   inst_IOBUF34 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(18),   -- Buffer output
      IO => user_io_u(18),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(18),  -- Buffer input
      T => user_io_dir(18)   -- 3-state enable input 
   );

   inst_IOBUF35 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(19),   -- Buffer output
      IO => user_io_u(19),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(19),  -- Buffer input
      T => user_io_dir(19)   -- 3-state enable input 
   );

   inst_IOBUF36 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(20),   -- Buffer output
      IO => user_io_u(20),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(20),  -- Buffer input
      T => user_io_dir(20)   -- 3-state enable input 
   );

   inst_IOBUF37 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(21),   -- Buffer output
      IO => user_io_u(21),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(21),  -- Buffer input
      T => user_io_dir(21)   -- 3-state enable input 
   );

   inst_IOBUF38 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(22),   -- Buffer output
      IO => user_io_u(22),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(22),  -- Buffer input
      T => user_io_dir(22)   -- 3-state enable input 
   );

   inst_IOBUF39 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(23),   -- Buffer output
      IO => user_io_u(23),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(23),  -- Buffer input
      T => user_io_dir(23)   -- 3-state enable input 
   );

   inst_IOBUF40 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(24),   -- Buffer output
      IO => user_io_u(24),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(24),  -- Buffer input
      T => user_io_dir(24)   -- 3-state enable input 
   );

   inst_IOBUF41 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(25),   -- Buffer output
      IO => user_io_u(25),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(25),  -- Buffer input
      T => user_io_dir(25)   -- 3-state enable input 
   );

   inst_IOBUF42 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(26),   -- Buffer output
      IO => user_io_u(26),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(26),  -- Buffer input
      T => user_io_dir(26)   -- 3-state enable input 
   );

   inst_IOBUF43 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(27),   -- Buffer output
      IO => user_io_u(27),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(27),  -- Buffer input
      T => user_io_dir(27)   -- 3-state enable input 
   );

   inst_IOBUF44 : IOBUF
   generic map (
      DRIVE => 12,
      IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E only)
      IFD_DELAY_VALUE => "AUTO", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E only)
      IOSTANDARD => "LVCMOS33",
      SLEW => "SLOW")
   port map (
      O => user_io_in(28),   -- Buffer output
      IO => user_io_u(28),   -- Buffer inout port (connect directly to top-level port)
      I => user_io_out(28),  -- Buffer input
      T => user_io_dir(28)   -- 3-state enable input 
   );

end Behavioral;

