-------------------------------------------------------------------------------
-- DISCLAIMER OF LIABILITY
--
-- DLP Design is providing this design, code, or information "as-is" solely for 
-- use in developing programs and solutions for DLP Design products, with no 
-- obligation on the part of DLP Design to provide support. DLP Design expressly 
-- disclaims any warranty whatsoever with respect to the adequacy of the 
-- implementation, including but not limited to any warranties or representations 
-- that this implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular purpose.
--
-- DLP Design products are not intended for use in life support appliances, devices, 
-- or systems. Use in such applications is expressly prohibited.
--
-- Any modifications that are made to the Source Code are done at the user's sole 
-- risk and will be unsupported.
--
-- Copyright (c) 2006-2009 DLP Design, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part of this text at all times.
----------------------------------------------------------------------------------
-- Company: DLP Design, Inc., www.dlpdesign.com
-- 
-- Create Date:    22:59:53 04/05/2006 
-- Design Name: 	 heartbeat	
-- Module Name:    heartbeat - Behavioral 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S250E-4TQ144A
-- Tool versions:  ISE 9.1.03i
-- Description:    aprox half second led strobe for mfg test design
--                 
-- Dependencies:   none
--
-- Revision: 
-- 	0.01 	04/05/06	File Created for heartbeat function
--    1.00  07/31/07	Final release
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity heartbeat is
    Port ( clk_in : in  STD_LOGIC;
           rst_in : in  STD_LOGIC;
           ledr_heartbeat : out  STD_LOGIC);
end heartbeat;

architecture rtl of heartbeat is

	signal ledr_heartbeat_int : std_logic;
	signal count : std_logic_vector(1 downto 0);

begin

	process (clk_in, rst_in) 
	begin
		if rst_in ='1' then 
			ledr_heartbeat_int <= '1';
			count <= (others => '0');
		elsif clk_in ='1' and clk_in'event then
			count <= count + 1;
 		   --ledr_heartbeat_int <= count(1); -- 1.2 sec on, 1.2 sec off
 		   ledr_heartbeat_int <= count(0); -- 0.6 sec on, 0.6 sec off
		end if;
	end process;

  ledr_heartbeat <= ledr_heartbeat_int;
	

end rtl;

