-------------------------------------------------------------------------------
-- DISCLAIMER OF LIABILITY
--
-- DLP Design is providing this design, code, or information "as-is" solely for 
-- use in developing programs and solutions for DLP Design products, with no 
-- obligation on the part of DLP Design to provide support. DLP Design expressly 
-- disclaims any warranty whatsoever with respect to the adequacy of the 
-- implementation, including but not limited to any warranties or representations 
-- that this implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular purpose.
--
-- DLP Design products are not intended for use in life support appliances, devices, 
-- or systems. Use in such applications is expressly prohibited.
--
-- Any modifications that are made to the Source Code are done at the user's sole 
-- risk and will be unsupported.
--
-- Copyright (c) 2006-2009 DLP Design, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part of this text at all times.
----------------------------------------------------------------------------------
-- Company: DLP Design, Inc., www.dlpdesign.com
-- 
-- Create Date:    22:37:36 04/12/2006 
-- Design Name: 	 startup	
-- Module Name:    startup - rtl 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S250E-4TQ144A
-- Tool versions:  ISE 9.1.03i
-- Description:    generates strobe on reset deassert, also debounce switches (not used)
--                 
-- Dependencies:   none
--
-- Revision: 
-- 	0.01 	04/12/06	File Created
--    1.00  07/31/07	Final release
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity startup is
    Port ( clk_in : in  STD_LOGIC;
           rst_in : in  STD_LOGIC;
           switch : in  STD_LOGIC_VECTOR (7 downto 0);
           start : out  STD_LOGIC_VECTOR (7 downto 0));
end startup;

architecture rtl of startup is

	signal rst_in_d, rst_in_d2 : std_logic;
	signal switch_d : std_logic_vector(7 downto 0); 
	signal switch_d2 : std_logic_vector(7 downto 0);
	signal start_p : std_logic_vector(7 downto 0);

begin

	edge_detect : process(clk_in)
	begin
      if (clk_in'event and clk_in = '1') then
         rst_in_d <= rst_in;
			rst_in_d2 <= rst_in_d;
			switch_d <= switch;
			switch_d2 <= switch_d;
			-- start strobe asserts high one clock on reset deassert
			--             or on control lsb bit assert high
			start_p(0) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(0) AND NOT(switch_d2(0)))); -- ctl lsb assert
			start_p(1) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(1) AND NOT(switch_d2(1)))); -- ctl lsb assert
			start_p(2) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(2) AND NOT(switch_d2(2)))); -- ctl lsb assert
			start_p(3) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(3) AND NOT(switch_d2(3)))); -- ctl lsb assert
			start_p(4) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(4) AND NOT(switch_d2(4)))); -- ctl lsb assert
			start_p(5) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(5) AND NOT(switch_d2(5)))); -- ctl lsb assert
			start_p(6) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(6) AND NOT(switch_d2(6)))); -- ctl lsb assert
			start_p(7) <= ((rst_in_d2 AND NOT(rst_in_d)) or -- reset deassert
			               (switch_d(7) AND NOT(switch_d2(7)))); -- ctl lsb assert
			start <= start_p; --add another clock of seperation after reset
		end if;
	end process;	

end rtl;

