-------------------------------------------------------------------------------
-- DISCLAIMER OF LIABILITY
--
-- DLP Design is providing this design, code, or information "as-is" solely for 
-- use in developing programs and solutions for DLP Design products, with no 
-- obligation on the part of DLP Design to provide support. DLP Design expressly 
-- disclaims any warranty whatsoever with respect to the adequacy of the 
-- implementation, including but not limited to any warranties or representations 
-- that this implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular purpose.
--
-- DLP Design products are not intended for use in life support appliances, devices, 
-- or systems. Use in such applications is expressly prohibited.
--
-- Any modifications that are made to the Source Code are done at the user's sole 
-- risk and will be unsupported.
--
-- Copyright (c) 2006-2009 DLP Design, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part of this text at all times.
----------------------------------------------------------------------------------
-- Company: DLP Design, Inc., www.dlpdesign.com
-- 
-- Create Date:    22:45:29 04/10/2006 
-- Design Name: 	 timer	
-- Module Name:    timer - Behavioral 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S250E-4TQ144A
-- Tool versions:  ISE 9.1.03i
-- Description:    divides down 400Khz clock to ~1/2 sec clock
--                 
-- Dependencies:   none
--
-- Revision: 
-- 	0.01 	04/10/06	File Created
--    1.00  07/31/07	Final release
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity timer is
    Port ( clkin_2u5s : in  STD_LOGIC;
	     ce_in : in std_logic;
           rst_in : in  STD_LOGIC;
           clk_10us : out  STD_LOGIC;
           clk_80us : out  STD_LOGIC;
           clk_1m28s : out  STD_LOGIC;
           clk_5m12s : out  STD_LOGIC;
           clk_10m24s : out  STD_LOGIC;
           clk_40m96s : out  STD_LOGIC;
           clk_81m92s : out  STD_LOGIC;
           clk_655m36s : out  STD_LOGIC);
end timer;

architecture rtl of timer is

	signal count : std_logic_vector(17 downto 0);

begin

	-- clock input period is 2.5 us or 400 KHz
	-- count 0 = 5 us period, or 200 KHz
	-- count 1 = 10 us period, or 100 KHz
	-- count 2 = 20 us period, or 50 KHz
	-- count 3 = 40 us period, or 25 KHz
	-- count 4 = 80 us period, or 12.5 KHz
	-- count 5 = 160 us period, or 6.25 KHz
	-- count 6 = 320 us period, or 3.125 kHz
	-- count 7 = 640 us period, or 1.5625 KHz
	-- count 8 = 1.28 ms period, or 781.25 Hz
	-- count 9 = 2.56 ms period, or 390.625 Hz
	-- count 10 = 5.12 ms period, or 195.3125 Hz
	-- count 11 = 10.24 ms period, or 97.65625 Hz
	-- count 12 = 20.48 ms period, or 48.828125 Hz
	-- count 13 = 40.96 ms period, or 24.4140625 Hz
	-- count 14 = 81.92 ms period, or 12.20703125 Hz
	-- count 15 = 163.84 ms period, or 6.103515625 Hz
	-- count 16 = 327.68 ms period, or 3.0517578125 Hz
	-- count 17 = 655.36 ms period, or 1.52587890625 Hz

	process (clkin_2u5s) 
	begin
		if clkin_2u5s ='1' and clkin_2u5s'event then
			if rst_in='1' then 
				count <= (others => '0');
			elsif ce_in='1' then
				count <= count + 1;
			end if;
			clk_10us <= count(1); --exact
			clk_80us <= count(4);
			clk_1m28s <= count(8);
			clk_5m12s <= count(10);
			clk_10m24s <= count(11);		
			clk_40m96s <= count(13);
			clk_81m92s <= count(14);
			clk_655m36s <= count(17);
		end if;
	end process; 

end rtl;

